@@ -38,7 +38,11 @@
 #include "applications-view.h"
 #include "windows-view.h"
 #include "search-view.h"
+#include "search-manager.h"
+#include "applications-search-provider.h"
 #include "utils.h"
+#include "theme.h"
+#include "focus-manager.h"
 
 /* Define this class in GObject system */
 G_DEFINE_TYPE(XfdashboardApplication,
@@ -59,7 +63,9 @@ struct _XfdashboardApplicationPrivate
 	gboolean					inited;
 	XfconfChannel				*xfconfChannel;
 	XfdashboardViewManager		*viewManager;
-	XfdashboardThemeCSS			*theme;
+	XfdashboardSearchManager	*searchManager;
+	XfdashboardFocusManager		*focusManager;
+	XfdashboardTheme			*theme;
 };
 
 /* Properties */
@@ -89,13 +95,12 @@ enum
 static guint XfdashboardApplicationSignals[SIGNAL_LAST]={ 0, };
 
 /* IMPLEMENTATION: Private variables and methods */
-#define XFDASHBOARD_APP_ID				"de.froevel.nomad.xfdashboard"
-#define XFDASHBOARD_XFCONF_CHANNEL		"xfdashboard"
+#define XFDASHBOARD_APP_ID					"de.froevel.nomad.xfdashboard"
+#define XFDASHBOARD_XFCONF_CHANNEL			"xfdashboard"
 
-#define THEME_NAME_XFCONF_PROP			"/theme"
-#define XFDASHBOARD_THEME_SUBPATH		"xfdashboard-1.0"
-#define XFDASHBOARD_THEME_CSS_FILE		"xfdashboard.css"
-#define DEFAULT_THEME_NAME				"xfdashboard"
+#define THEME_NAME_XFCONF_PROP				"/theme"
+#define DEFAULT_THEME_NAME					"xfdashboard"
+#define XFDASHBOARD_THEME_LAYOUT_PRIMARY	"primary"
 
 /* Single instance of application */
 static XfdashboardApplication*		application=NULL;
@@ -167,15 +172,13 @@ static gboolean _xfdashboard_application_load_theme(XfdashboardApplication *self
 	XfdashboardApplicationPrivate	*priv;
 	GError							*error;
 	gchar							*themeName;
-	gchar							*themeFile;
-	XfdashboardThemeCSS				*theme;
+	XfdashboardTheme				*theme;
 
 	g_return_val_if_fail(XFDASHBOARD_IS_APPLICATION(self), FALSE);
 
 	priv=self->priv;
 	error=NULL;
 	themeName=NULL;
-	themeFile=NULL;
 	theme=NULL;
 
 	/* Determine theme file to load and check if file exists */
@@ -188,90 +191,14 @@ static gboolean _xfdashboard_application_load_theme(XfdashboardApplication *self
 		return(FALSE);
 	}
 
-#ifdef DEBUG
-	/* If debug is enabled search in project's data dir first assuming
-	 * that current path is project's top level dir.
-	 * That means we look for: "./data/themes/<<theme name>>/xfdashboard.css" */
-	if(!themeFile)
-	{
-		gchar						*currentPath;
-
-		currentPath=g_get_current_dir();
-		if(currentPath)
-		{
-			themeFile=g_build_filename(currentPath, "data", "themes", themeName, XFDASHBOARD_THEME_CSS_FILE, NULL);
-			g_debug("Trying theme file: %s", themeFile);
-			if(!g_file_test(themeFile, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR))
-			{
-				g_free(themeFile);
-				themeFile=NULL;
-			}
-
-			g_free(currentPath);
-		}
-	}
-#endif
-
-	/* Search theme file in user's config dir first */
-	if(!themeFile)
-	{
-		themeFile=g_build_filename(g_get_user_data_dir(), "themes", themeName, XFDASHBOARD_THEME_SUBPATH, XFDASHBOARD_THEME_CSS_FILE, NULL);
-		g_debug("Trying theme file: %s", themeFile);
-		if(!g_file_test(themeFile, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR))
-		{
-			g_free(themeFile);
-			themeFile=NULL;
-		}
-	}
-
-	/* If file not found search in user's home directory */
-	if(!themeFile)
-	{
-		const gchar					*homeDirectory;
-
-		homeDirectory=g_get_home_dir();
-		if(homeDirectory)
-		{
-			themeFile=g_build_filename(homeDirectory, ".themes", themeName, XFDASHBOARD_THEME_SUBPATH, XFDASHBOARD_THEME_CSS_FILE, NULL);
-			g_debug("Trying theme file: %s", themeFile);
-			if(!g_file_test(themeFile, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR))
-			{
-				g_free(themeFile);
-				themeFile=NULL;
-			}
-		}
-	}
-
-	/* If file not found search in system-wide paths */
-	if(!themeFile)
-	{
-		themeFile=g_build_filename(PACKAGE_DATADIR, "themes", themeName, XFDASHBOARD_THEME_SUBPATH, XFDASHBOARD_THEME_CSS_FILE, NULL);
-		g_debug("Trying theme file: %s", themeFile);
-		if(!g_file_test(themeFile, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR))
-		{
-			g_free(themeFile);
-			themeFile=NULL;
-		}
-	}
-
-	/* If file is still not found we cannot load theme */
-	if(!themeFile)
-	{
-		g_critical(_("Could not find theme files for theme '%s'!"), themeName);
-		g_free(themeName);
-		return(FALSE);
-	}
-
 	/* Create new theme instance and load theme */
-	theme=xfdashboard_theme_css_new();
-	if(!xfdashboard_theme_css_add_file(theme, themeFile, 0, &error))
+	theme=xfdashboard_theme_new();
+	if(!xfdashboard_theme_load(theme, themeName, &error))
 	{
-		g_critical(_("Could not load file '%s' of theme '%s': %s"),
-					themeFile,
+		g_critical(_("Could not load theme '%s': %s"),
 					themeName,
 					(error && error->message) ? error->message : _("unknown error"));
 		if(error!=NULL) g_error_free(error);
-		g_free(themeFile);
 		g_free(themeName);
 		g_object_unref(theme);
 		return(FALSE);
@@ -287,7 +214,6 @@ static gboolean _xfdashboard_application_load_theme(XfdashboardApplication *self
 	priv->theme=theme;
 
 	/* Release allocated resources */
-	g_free(themeFile);
 	g_free(themeName);
 
 	return(TRUE);
@@ -299,6 +225,7 @@ static gboolean _xfdashboard_application_initialize_full(XfdashboardApplication
 	XfdashboardApplicationPrivate	*priv;
 	GError							*error;
 	ClutterActor					*stage;
+	XfdashboardThemeLayout			*themeLayout;
 
 	g_return_val_if_fail(XFDASHBOARD_IS_APPLICATION(self), FALSE);
 
@@ -319,19 +246,46 @@ static gboolean _xfdashboard_application_initialize_full(XfdashboardApplication
 	/* Load theme */
 	if(!_xfdashboard_application_load_theme(self)) return(FALSE);
 
-	/* Register views (order of registration is important) */
+	/* Register built-in views (order of registration is important) */
 	priv->viewManager=xfdashboard_view_manager_get_default();
 
 	xfdashboard_view_manager_register(priv->viewManager, XFDASHBOARD_TYPE_WINDOWS_VIEW);
 	xfdashboard_view_manager_register(priv->viewManager, XFDASHBOARD_TYPE_APPLICATIONS_VIEW);
 	xfdashboard_view_manager_register(priv->viewManager, XFDASHBOARD_TYPE_SEARCH_VIEW);
 
+	/* Register built-in search providers */
+	priv->searchManager=xfdashboard_search_manager_get_default();
+
+	xfdashboard_search_manager_register(priv->searchManager, XFDASHBOARD_TYPE_APPLICATIONS_SEARCH_PROVIDER);
+
+	/* Create single-instance of focus manager to keep it alive while
+	 * application is running.
+	 */
+	priv->focusManager=xfdashboard_focus_manager_get_default();
+
 	/* Create primary stage on first monitor
 	 * TODO: Create stage for each monitor connected
 	 *       but only primary monitor gets its stage
 	 *       setup for primary display
 	 */
-	stage=xfdashboard_stage_new();
+	themeLayout=xfdashboard_theme_get_layout(priv->theme);
+	stage=xfdashboard_theme_layout_build_interface(themeLayout, XFDASHBOARD_THEME_LAYOUT_PRIMARY);
+	if(!stage)
+	{
+		g_critical(_("Could not build interface '%s' from theme '%s'"),
+					XFDASHBOARD_THEME_LAYOUT_PRIMARY,
+					xfdashboard_theme_get_theme_name(priv->theme));
+		return(FALSE);
+	}
+
+	if(!XFDASHBOARD_IS_STAGE(stage))
+	{
+		g_critical(_("Interface '%s' from theme '%s' must be an actor of type %s"),
+					XFDASHBOARD_THEME_LAYOUT_PRIMARY,
+					xfdashboard_theme_get_theme_name(priv->theme),
+					g_type_name(XFDASHBOARD_TYPE_STAGE));
+		return(FALSE);
+	}
 
 	if(!priv->isDaemon) clutter_actor_show(stage);
 	g_signal_connect_swapped(stage, "delete-event", G_CALLBACK(_xfdashboard_application_on_delete_stage), self);
@@ -429,7 +383,7 @@ static int _xfdashboard_application_command_line(GApplication *inApplication, GA
 	if(optionQuit)
 	{
 		/* Quit existing instance */
-		g_debug(_("Quitting running instance!"));
+		g_debug("Quitting running instance!");
 		_xfdashboard_application_quit(self, TRUE);
 
 		return(XFDASHBOARD_APPLICATION_ERROR_QUIT);
@@ -491,6 +445,22 @@ static void _xfdashboard_application_dispose(GObject *inObject)
 		priv->viewManager=NULL;
 	}
 
+	if(priv->searchManager)
+	{
+		/* Unregisters all remaining registered providers - no need to unregister them here */
+		g_object_unref(priv->searchManager);
+		priv->searchManager=NULL;
+	}
+
+	if(priv->focusManager)
+	{
+		/* Unregisters all remaining registered focusable actors.
+		 * There is no need to unregister them here.
+		 */
+		g_object_unref(priv->focusManager);
+		priv->focusManager=NULL;
+	}
+
 	/* Shutdown xfconf */
 	priv->xfconfChannel=NULL;
 	xfconf_shutdown();
@@ -565,14 +535,14 @@ static void xfdashboard_application_class_init(XfdashboardApplicationClass *klas
 								_("is-daemonized"),
 								_("Flag indicating if application is daemonized"),
 								FALSE,
-								G_PARAM_READABLE);
+								G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
 	XfdashboardApplicationProperties[PROP_SUSPENDED]=
 		g_param_spec_boolean("is-suspended",
 								_("Is suspended"),
 								_("Flag indicating if application is suspended currently"),
 								FALSE,
-								G_PARAM_READABLE);
+								G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
 	g_object_class_install_properties(gobjectClass, PROP_LAST, XfdashboardApplicationProperties);
 
@@ -620,6 +590,9 @@ static void xfdashboard_application_class_init(XfdashboardApplicationClass *klas
 						g_cclosure_marshal_VOID__VOID,
 						G_TYPE_NONE,
 						0);
+
+	/* Register GValue transformation function not provided by any other library */
+	xfdashboard_register_gvalue_transformation_funcs();
 }
 
 /* Object initialization
@@ -637,6 +610,8 @@ static void xfdashboard_application_init(XfdashboardApplication *self)
 	priv->inited=FALSE;
 	priv->xfconfChannel=NULL;
 	priv->viewManager=NULL;
+	priv->searchManager=NULL;
+	priv->focusManager=NULL;
 	priv->theme=NULL;
 }
 
@@ -701,9 +676,9 @@ XfconfChannel* xfdashboard_application_get_xfconf_channel(void)
 }
 
 /* Get current theme used */
-XfdashboardThemeCSS* xfdashboard_application_get_theme(void)
+XfdashboardTheme* xfdashboard_application_get_theme(void)
 {
-	XfdashboardThemeCSS		*theme=NULL;
+	XfdashboardTheme		*theme=NULL;
 
 	if(G_LIKELY(application!=NULL)) theme=application->priv->theme;
 
