@@ -40,11 +40,12 @@
 #include "click-action.h"
 #include "window-content.h"
 #include "image.h"
+#include "stylable.h"
 
 /* Define this class in GObject system */
 G_DEFINE_TYPE(XfdashboardLiveWindow,
 				xfdashboard_live_window,
-				XFDASHBOARD_TYPE_ACTOR)
+				XFDASHBOARD_TYPE_BACKGROUND)
 
 /* Private structure - access only by public API if needed */
 #define XFDASHBOARD_LIVE_WINDOW_GET_PRIVATE(obj) \
@@ -610,7 +611,7 @@ static void xfdashboard_live_window_class_init(XfdashboardLiveWindowClass *klass
 								_("Window"),
 								_("The window to show"),
 								XFDASHBOARD_TYPE_WINDOW_TRACKER_WINDOW,
-								G_PARAM_READWRITE);
+								G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
 	XfdashboardLiveWindowProperties[PROP_CLOSE_BUTTON_PADDING]=
 		g_param_spec_float("close-padding",
@@ -618,7 +619,7 @@ static void xfdashboard_live_window_class_init(XfdashboardLiveWindowClass *klass
 							_("Padding of close button to window actor in pixels"),
 							0.0f, G_MAXFLOAT,
 							0.0f,
-							G_PARAM_READWRITE);
+							G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
 	XfdashboardLiveWindowProperties[PROP_TITLE_ACTOR_PADDING]=
 		g_param_spec_float("title-padding",
@@ -626,7 +627,7 @@ static void xfdashboard_live_window_class_init(XfdashboardLiveWindowClass *klass
 							_("Padding of title actor to window actor in pixels"),
 							0.0f, G_MAXFLOAT,
 							0.0f,
-							G_PARAM_READWRITE);
+							G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
 	g_object_class_install_properties(gobjectClass, PROP_LAST, XfdashboardLiveWindowProperties);
 
@@ -717,13 +718,13 @@ static void xfdashboard_live_window_init(XfdashboardLiveWindow *self)
 	clutter_actor_add_child(CLUTTER_ACTOR(self), priv->actorWindow);
 
 	priv->actorTitle=xfdashboard_button_new();
-	xfdashboard_actor_add_style_class(XFDASHBOARD_ACTOR(priv->actorTitle), "title");
+	xfdashboard_stylable_add_class(XFDASHBOARD_STYLABLE(priv->actorTitle), "title");
 	clutter_actor_set_reactive(priv->actorTitle, FALSE);
 	clutter_actor_show(priv->actorTitle);
 	clutter_actor_add_child(CLUTTER_ACTOR(self), priv->actorTitle);
 
 	priv->actorClose=xfdashboard_button_new();
-	xfdashboard_actor_add_style_class(XFDASHBOARD_ACTOR(priv->actorClose), "close-button");
+	xfdashboard_stylable_add_class(XFDASHBOARD_STYLABLE(priv->actorClose), "close-button");
 	clutter_actor_set_reactive(priv->actorClose, FALSE);
 	clutter_actor_show(priv->actorClose);
 	clutter_actor_add_child(CLUTTER_ACTOR(self), priv->actorClose);
