@@ -54,6 +54,7 @@ struct _XfdashboardScaledTableLayoutPrivate
 	/* Instance related */
 	gint		rows;
 	gint		columns;
+	gint		numberChildren;
 };
 
 /* Properties */
@@ -65,6 +66,10 @@ enum
 	PROP_COLUMN_SPACING,
 	PROP_RELATIVE_SCALE,
 
+	PROP_NUMBER_CHILDREN,
+	PROP_ROWS,
+	PROP_COLUMNS,
+
 	PROP_LAST
 };
 
@@ -77,10 +82,12 @@ static void _xfdashboard_scaled_table_layout_update_rows_and_columns(Xfdashboard
 																		ClutterContainer *inContainer)
 {
 	XfdashboardScaledTableLayoutPrivate		*priv;
-	ClutterActor							*child;
-	ClutterActorIter						iter;
 	ClutterRequestMode						requestMode;
+	ClutterActorIter						iter;
+	ClutterActor							*child;
 	gint									numberChildren;
+	gint									rows;
+	gint									columns;
 
 	g_return_if_fail(XFDASHBOARD_IS_SCALED_TABLE_LAYOUT(self));
 	g_return_if_fail(CLUTTER_IS_CONTAINER(inContainer));
@@ -88,6 +95,9 @@ static void _xfdashboard_scaled_table_layout_update_rows_and_columns(Xfdashboard
 
 	priv=self->priv;
 
+	/* Freeze notification */
+	g_object_freeze_notify(G_OBJECT(self));
+
 	/* Get number of visible child actors */
 	numberChildren=0;
 	clutter_actor_iter_init(&iter, CLUTTER_ACTOR(inContainer));
@@ -96,6 +106,12 @@ static void _xfdashboard_scaled_table_layout_update_rows_and_columns(Xfdashboard
 		if(CLUTTER_ACTOR_IS_VISIBLE(child)) numberChildren++;
 	}
 
+	if(numberChildren!=priv->numberChildren)
+	{
+		priv->numberChildren=numberChildren;
+		g_object_notify_by_pspec(G_OBJECT(self), XfdashboardScaledTableLayoutProperties[PROP_NUMBER_CHILDREN]);
+	}
+
 	/* Get request mode to determine if more rows than colums are needed
 	 * or the opposite
 	 */
@@ -104,14 +120,29 @@ static void _xfdashboard_scaled_table_layout_update_rows_and_columns(Xfdashboard
 	/* Calculate and update number of rows and columns */
 	if(requestMode==CLUTTER_REQUEST_HEIGHT_FOR_WIDTH)
 	{
-		priv->rows=ceil(sqrt((double)numberChildren));
-		priv->columns=ceil((double)numberChildren / (double)priv->rows);
+		rows=ceil(sqrt((double)priv->numberChildren));
+		columns=ceil((double)priv->numberChildren / (double)priv->rows);
 	}
 		else
 		{
-			priv->columns=ceil(sqrt((double)numberChildren));
-			priv->rows=ceil((double)numberChildren / (double)priv->columns);
+			columns=ceil(sqrt((double)priv->numberChildren));
+			rows=ceil((double)priv->numberChildren / (double)priv->columns);
 		}
+
+	if(rows!=priv->rows)
+	{
+		priv->rows=rows;
+		g_object_notify_by_pspec(G_OBJECT(self), XfdashboardScaledTableLayoutProperties[PROP_ROWS]);
+	}
+
+	if(columns!=priv->columns)
+	{
+		priv->columns=columns;
+		g_object_notify_by_pspec(G_OBJECT(self), XfdashboardScaledTableLayoutProperties[PROP_COLUMNS]);
+	}
+
+	/* Thaw notification */
+	g_object_thaw_notify(G_OBJECT(self));
 }
 
 /* IMPLEMENTATION: ClutterLayoutManager */
@@ -360,6 +391,18 @@ static void _xfdashboard_scaled_table_layout_get_property(GObject *inObject,
 			g_value_set_boolean(outValue, self->priv->relativeScale);
 			break;
 
+		case PROP_NUMBER_CHILDREN:
+			g_value_set_int(outValue, self->priv->numberChildren);
+			break;
+
+		case PROP_ROWS:
+			g_value_set_int(outValue, self->priv->rows);
+			break;
+
+		case PROP_COLUMNS:
+			g_value_set_int(outValue, self->priv->columns);
+			break;
+
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID(inObject, inPropID, inSpec);
 			break;
@@ -394,7 +437,7 @@ static void xfdashboard_scaled_table_layout_class_init(XfdashboardScaledTableLay
 								0.0f,
 								G_MAXFLOAT,
 								0.0f,
-								G_PARAM_READWRITE);
+								G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
 	XfdashboardScaledTableLayoutProperties[PROP_COLUMN_SPACING]=
 		g_param_spec_float("column-spacing",
@@ -403,15 +446,42 @@ static void xfdashboard_scaled_table_layout_class_init(XfdashboardScaledTableLay
 								0.0f,
 								G_MAXFLOAT,
 								0.0f,
-								G_PARAM_READWRITE);
+								G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
 	XfdashboardScaledTableLayoutProperties[PROP_RELATIVE_SCALE]=
 		g_param_spec_boolean("relative-scale",
 								_("Relative scale"),
 								_("Whether all children should be scaled relatively to largest child"),
 								FALSE,
-								G_PARAM_READWRITE);
+								G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
+	XfdashboardScaledTableLayoutProperties[PROP_NUMBER_CHILDREN]=
+		g_param_spec_float("number-children",
+								_("Number children"),
+								_("Current number of child actors in this layout"),
+								0,
+								G_MAXINT,
+								0,
+								G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+
+	XfdashboardScaledTableLayoutProperties[PROP_ROWS]=
+		g_param_spec_float("rows",
+								_("Rows"),
+								_("Current number of rows in this layout"),
+								0,
+								G_MAXINT,
+								0,
+								G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+
+	XfdashboardScaledTableLayoutProperties[PROP_COLUMNS]=
+		g_param_spec_float("columns",
+								_("Columns"),
+								_("Current number of columns in this layout"),
+								0,
+								G_MAXINT,
+								0,
+								G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+	
 	g_object_class_install_properties(gobjectClass, PROP_LAST, XfdashboardScaledTableLayoutProperties);
 }
 
@@ -431,6 +501,7 @@ static void xfdashboard_scaled_table_layout_init(XfdashboardScaledTableLayout *s
 
 	priv->rows=0;
 	priv->columns=0;
+	priv->numberChildren=0;
 }
 
 /* IMPLEMENTATION: Public API */
@@ -441,6 +512,30 @@ ClutterLayoutManager* xfdashboard_scaled_table_layout_new(void)
 	return(CLUTTER_LAYOUT_MANAGER(g_object_new(XFDASHBOARD_TYPE_SCALED_TABLE_LAYOUT, NULL)));
 }
 
+/* Get number of (visible) children which will be layouted */
+gint xfdashboard_scaled_table_layout_get_number_children(XfdashboardScaledTableLayout *self)
+{
+	g_return_val_if_fail(XFDASHBOARD_IS_SCALED_TABLE_LAYOUT(self), 0);
+
+	return(self->priv->numberChildren);
+}
+
+/* Get number of rows */
+gint xfdashboard_scaled_table_layout_get_rows(XfdashboardScaledTableLayout *self)
+{
+	g_return_val_if_fail(XFDASHBOARD_IS_SCALED_TABLE_LAYOUT(self), 0);
+
+	return(self->priv->rows);
+}
+
+/* Get number of columns */
+gint xfdashboard_scaled_table_layout_get_columns(XfdashboardScaledTableLayout *self)
+{
+	g_return_val_if_fail(XFDASHBOARD_IS_SCALED_TABLE_LAYOUT(self), 0);
+
+	return(self->priv->columns);
+}
+
 /* Get/set relative scaling of all children to largest one */
 gboolean xfdashboard_scaled_table_layout_get_relative_scale(XfdashboardScaledTableLayout *self)
 {
