@@ -30,6 +30,8 @@
 #include <glib/gi18n-lib.h>
 #include <clutter/clutter.h>
 #include <gtk/gtk.h>
+#include <gdk/gdk.h>
+#include <math.h>
 
 #include "application.h"
 #include "viewpad.h"
@@ -42,16 +44,21 @@
 #include "toggle-button.h"
 #include "workspace-selector.h"
 #include "collapse-box.h"
-
-/* TODO: Replace by settings/theming object (or check monitor directions if more than one monitor)
- * Meanwhile #undef STAGE_USE_HORIZONTAL_WORKSPACE_SELECTOR for vertical workspace
- */
-#undef STAGE_USE_HORIZONTAL_WORKSPACE_SELECTOR
+#include "tooltip-action.h"
+#include "layoutable.h"
+#include "stylable.h"
+#include "utils.h"
+#include "focus-manager.h"
 
 /* Define this class in GObject system */
-G_DEFINE_TYPE(XfdashboardStage,
-				xfdashboard_stage,
-				CLUTTER_TYPE_STAGE)
+static void _xfdashboard_stage_layoutable_iface_init(XfdashboardLayoutableInterface *iface);
+static void _xfdashboard_stage_stylable_iface_init(XfdashboardStylableInterface *iface);
+
+G_DEFINE_TYPE_WITH_CODE(XfdashboardStage,
+						xfdashboard_stage,
+						CLUTTER_TYPE_STAGE,
+						G_IMPLEMENT_INTERFACE(XFDASHBOARD_TYPE_LAYOUTABLE, _xfdashboard_stage_layoutable_iface_init)
+						G_IMPLEMENT_INTERFACE(XFDASHBOARD_TYPE_STYLABLE, _xfdashboard_stage_stylable_iface_init))
 
 /* Private structure - access only by public API if needed */
 #define XFDASHBOARD_STAGE_GET_PRIVATE(obj) \
@@ -59,6 +66,10 @@ G_DEFINE_TYPE(XfdashboardStage,
 
 struct _XfdashboardStagePrivate
 {
+	/* Properties related */
+	gchar							*styleClasses;
+	gchar							*stylePseudoClasses;
+
 	/* Actors */
 	ClutterActor					*quicklaunch;
 	ClutterActor					*searchbox;
@@ -66,6 +77,7 @@ struct _XfdashboardStagePrivate
 	ClutterActor					*viewpad;
 	ClutterActor					*viewSelector;
 	ClutterActor					*notification;
+	ClutterActor					*tooltip;
 
 	/* Instance related */
 	XfdashboardWindowTracker		*windowTracker;
@@ -76,6 +88,19 @@ struct _XfdashboardStagePrivate
 	XfdashboardView					*viewBeforeSearch;
 
 	guint							notificationTimeoutID;
+
+	XfdashboardFocusManager			*focusManager;
+};
+
+/* Properties */
+enum
+{
+	PROP_0,
+
+	PROP_STYLE_CLASSES,
+	PROP_STYLE_PSEUDO_CLASSES,
+
+	PROP_LAST
 };
 
 /* Signals */
@@ -85,6 +110,8 @@ enum
 	SIGNAL_SEARCH_CHANGED,
 	SIGNAL_SEARCH_ENDED,
 
+	SIGNAL_SHOW_TOOLTIP,
+
 	SIGNAL_LAST
 };
 
@@ -92,13 +119,231 @@ static guint XfdashboardStageSignals[SIGNAL_LAST]={ 0, };
 
 
 /* IMPLEMENTATION: Private variables and methods */
-static ClutterColor		defaultStageColor={ 0x00, 0x00, 0x00, 0xe0 };					// TODO: Replace by settings/theming object
-	
 #define NOTIFICATION_TIMEOUT_XFCONF_PROP		"/min-notification-timeout"
 #define DEFAULT_NOTIFICATION_TIMEOUT			3000
 #define RESET_SEARCH_ON_RESUME_XFCONF_PROP		"/reset-search-on-resume"
 #define DEFAULT_RESET_SEARCH_ON_RESUME			TRUE
 
+/* Handle an event */
+static gboolean xfdashboard_stage_event(ClutterActor *inActor, ClutterEvent *inEvent)
+{
+	XfdashboardStage			*self;
+	XfdashboardStagePrivate		*priv;
+	gboolean					result;
+
+	g_return_val_if_fail(XFDASHBOARD_IS_STAGE(inActor), CLUTTER_EVENT_PROPAGATE);
+
+	self=XFDASHBOARD_STAGE(inActor);
+	priv=self->priv;
+
+	/* Do only intercept any event if a focus manager is available */
+	if(!priv->focusManager) return(CLUTTER_EVENT_PROPAGATE);
+
+	/* Do only intercept "key-press" and "key-release" events */
+	if(clutter_event_type(inEvent)!=CLUTTER_KEY_PRESS &&
+		clutter_event_type(inEvent)!=CLUTTER_KEY_RELEASE)
+	{
+		return(CLUTTER_EVENT_PROPAGATE);
+	}
+
+	/* Handle key release event */
+	if(clutter_event_type(inEvent)==CLUTTER_KEY_RELEASE)
+	{
+		gboolean		isShift=FALSE;
+
+		/* Get modifiers we are interested in */
+		if(clutter_event_has_shift_modifier(inEvent)) isShift=TRUE;
+
+		/* Handle key */
+		switch(inEvent->key.keyval)
+		{
+			/* Handle TAB and SHIFT-TAB directly to move focus */
+			case CLUTTER_KEY_Tab:
+			case CLUTTER_KEY_ISO_Left_Tab:
+			{
+				XfdashboardFocusable	*currentFocusable;
+				XfdashboardFocusable	*newFocusable;
+
+				/* Get current focus */
+				currentFocusable=xfdashboard_focus_manager_get_focus(priv->focusManager);
+
+				/* If TAB is pressed without SHIFT move focus to next focusable actor.
+				 * If TAB is pressed with SHIFT move focus to previous focusable actor.
+				 */
+				if(!isShift)
+				{
+					newFocusable=xfdashboard_focus_manager_get_next_focusable(priv->focusManager, currentFocusable);
+					if(newFocusable) xfdashboard_focus_manager_set_focus(priv->focusManager, newFocusable);
+
+					return(CLUTTER_EVENT_STOP);
+				}
+					else
+					{
+						newFocusable=xfdashboard_focus_manager_get_previous_focusable(priv->focusManager, currentFocusable);
+						if(newFocusable) xfdashboard_focus_manager_set_focus(priv->focusManager, newFocusable);
+
+						return(CLUTTER_EVENT_STOP);
+					}
+			}
+			break;
+
+			/* Handle ESC key to clear search box or quit/suspend application */
+			case CLUTTER_KEY_Escape:
+			{
+				/* If search is active then end search by clearing search box ... */
+				if(priv->searchbox &&
+					!xfdashboard_text_box_is_empty(XFDASHBOARD_TEXT_BOX(priv->searchbox)))
+				{
+					xfdashboard_text_box_set_text(XFDASHBOARD_TEXT_BOX(priv->searchbox), NULL);
+					return(CLUTTER_EVENT_STOP);
+				}
+					/* ... otherwise quit application */
+					else
+					{
+						xfdashboard_application_quit();
+						return(CLUTTER_EVENT_STOP);
+					}
+			}
+			break;
+
+			default:
+				/* Fallthrough */
+				break;
+		}
+	}
+
+	/* Ask focus manager to handle this event */
+	result=xfdashboard_focus_manager_handle_key_event(priv->focusManager, inEvent);
+	if(result==CLUTTER_EVENT_STOP) return(result);
+
+	/* If even focus manager did not handle this event
+	 * send even to searchbox.
+	 */
+	if(priv->searchbox &&
+		XFDASHBOARD_IS_FOCUSABLE(priv->searchbox) &&
+		xfdashboard_focus_manager_is_registered(priv->focusManager, XFDASHBOARD_FOCUSABLE(priv->searchbox)))
+	{
+		result=xfdashboard_focusable_handle_key_event(XFDASHBOARD_FOCUSABLE(priv->searchbox), inEvent);
+		if(result==CLUTTER_EVENT_STOP) return(result);
+	}
+
+	/* If we get here there was no searchbox or it could not handle the event
+	 * so stop further processing.
+	 */
+	return(CLUTTER_EVENT_STOP);
+}
+
+/* Set focus in stage */
+static void _xfdashboard_stage_set_focus(XfdashboardStage *self)
+{
+	XfdashboardStagePrivate		*priv;
+	ClutterActor				*actor;
+
+	g_return_if_fail(XFDASHBOARD_IS_STAGE(self));
+
+	priv=self->priv;
+
+	/* Set focus if no focus is set */
+	actor=clutter_stage_get_key_focus(CLUTTER_STAGE(self));
+	if(!XFDASHBOARD_IS_FOCUSABLE(actor) ||
+		!xfdashboard_focus_manager_is_registered(priv->focusManager, XFDASHBOARD_FOCUSABLE(actor)))
+	{
+		XfdashboardFocusable	*focusable;
+
+		/* First try to set focus to searchbox ... */
+		if(XFDASHBOARD_IS_FOCUSABLE(priv->searchbox) &&
+			xfdashboard_focusable_can_focus(XFDASHBOARD_FOCUSABLE(priv->searchbox)))
+		{
+			xfdashboard_focus_manager_set_focus(priv->focusManager, XFDASHBOARD_FOCUSABLE(priv->searchbox));
+		}
+			/* ... then lookup first focusable actor */
+			else
+			{
+				focusable=xfdashboard_focus_manager_get_next_focusable(priv->focusManager, NULL);
+				if(focusable) xfdashboard_focus_manager_set_focus(priv->focusManager, focusable);
+			}
+	}
+}
+
+/* The pointer has been moved after a tooltip was shown */
+static gboolean _xfdashboard_stage_on_captured_event_after_tooltip(XfdashboardStage *self,
+																	ClutterEvent *inEvent,
+																	gpointer inUserData)
+{
+	XfdashboardStagePrivate		*priv;
+
+	g_return_val_if_fail(XFDASHBOARD_IS_STAGE(self), CLUTTER_EVENT_PROPAGATE);
+
+	priv=self->priv;
+
+	/* Disconnect signal */
+	g_signal_handlers_disconnect_by_func(self,
+											G_CALLBACK(_xfdashboard_stage_on_captured_event_after_tooltip),
+											NULL);
+
+	/* Hide tooltip */
+	clutter_actor_hide(priv->tooltip);
+
+	return(CLUTTER_EVENT_PROPAGATE);
+}
+
+/* Stage got signal to show a tooltip */
+static void _xfdashboard_stage_show_tooltip(XfdashboardStage *self, ClutterAction *inAction)
+{
+	XfdashboardStagePrivate		*priv;
+	XfdashboardTooltipAction	*tooltipAction;
+	const gchar					*tooltipText;
+	gfloat						tooltipX, tooltipY;
+	gfloat						tooltipWidth, tooltipHeight;
+	guint						cursorSize;
+	gfloat						x, y;
+	gfloat						stageWidth, stageHeight;
+
+	g_return_if_fail(XFDASHBOARD_IS_STAGE(self));
+	g_return_if_fail(XFDASHBOARD_IS_TOOLTIP_ACTION(inAction));
+	g_return_if_fail(self->priv->tooltip);
+
+	priv=self->priv;
+	tooltipAction=XFDASHBOARD_TOOLTIP_ACTION(inAction);
+
+	/* Disconnect any signal handler for hiding tooltip because
+	 * we are going to resetup tooltip. Hide tooltip while setup
+	 * to avoid flicker.
+	 */
+	g_signal_handlers_disconnect_by_func(self,
+											G_CALLBACK(_xfdashboard_stage_on_captured_event_after_tooltip),
+											NULL);
+	clutter_actor_hide(priv->tooltip);
+
+	/* Get tooltip text and update text in tooltip actor */
+	tooltipText=xfdashboard_tooltip_action_get_text(tooltipAction);
+	xfdashboard_text_box_set_text(XFDASHBOARD_TEXT_BOX(priv->tooltip), tooltipText);
+
+	/* Determine coordinates where to show tooltip at */
+	xfdashboard_tooltip_action_get_position(tooltipAction, &tooltipX, &tooltipY);
+	clutter_actor_get_size(priv->tooltip, &tooltipWidth, &tooltipHeight);
+
+	cursorSize=gdk_display_get_default_cursor_size(gdk_display_get_default());
+
+	clutter_actor_get_size(CLUTTER_ACTOR(self), &stageWidth, &stageHeight);
+
+	x=tooltipX+cursorSize;
+	y=tooltipY+cursorSize;
+	if((x+tooltipWidth)>stageWidth) x=tooltipX-tooltipWidth;
+	if((y+tooltipHeight)>stageHeight) y=tooltipY-tooltipHeight;
+
+	clutter_actor_set_position(priv->tooltip, floor(x), floor(y));
+
+	/* Show tooltip */
+	clutter_actor_show(priv->tooltip);
+
+	/* Connect signal to hide tooltip on next movement of pointer */
+	g_signal_connect(self,
+						"captured-event",
+						G_CALLBACK(_xfdashboard_stage_on_captured_event_after_tooltip),
+						NULL);
+}
+
 /* Notification timeout has been reached */
 static void _xfdashboard_stage_on_notification_timeout_destroyed(gpointer inUserData)
 {
@@ -131,39 +376,6 @@ static gboolean _xfdashboard_stage_on_notification_timeout(gpointer inUserData)
 	return(G_SOURCE_REMOVE);
 }
 
-/* A pressed key was released */
-static gboolean _xfdashboard_stage_on_key_release(XfdashboardStage *self,
-													ClutterEvent *inEvent,
-													gpointer inUserData)
-{
-	XfdashboardStagePrivate		*priv;
-
-	g_return_val_if_fail(XFDASHBOARD_IS_STAGE(self), CLUTTER_EVENT_PROPAGATE);
-
-	priv=self->priv;
-
-	/* Handle escape key */
-	if(inEvent->key.keyval==CLUTTER_Escape)
-	{
-		/* If search is active then end search by clearing search box ... */
-		if(priv->searchbox &&
-			!xfdashboard_text_box_is_empty(XFDASHBOARD_TEXT_BOX(priv->searchbox)))
-		{
-			xfdashboard_text_box_set_text(XFDASHBOARD_TEXT_BOX(priv->searchbox), NULL);
-			return(CLUTTER_EVENT_STOP);
-		}
-			/* ... otherwise quit application */
-			else
-			{
-				xfdashboard_application_quit();
-				return(CLUTTER_EVENT_STOP);
-			}
-	}
-
-	/* We did not handle this event so let next actor in clutter's chain handle it */
-	return(CLUTTER_EVENT_PROPAGATE);
-}
-
 /* App-button was toggled */
 static void _xfdashboard_stage_on_quicklaunch_apps_button_toggled(XfdashboardStage *self, gpointer inUserData)
 {
@@ -246,11 +458,10 @@ static void _xfdashboard_stage_on_searchbox_text_changed(XfdashboardStage *self,
 		xfdashboard_view_set_enabled(searchView, TRUE);
 
 		/* Activate "clear" button on text box */
-		xfdashboard_actor_add_style_class(XFDASHBOARD_ACTOR(priv->searchbox), "search-active");
+		xfdashboard_stylable_add_class(XFDASHBOARD_STYLABLE(priv->searchbox), "search-active");
 
 		/* Change apps button appearance */
-		// TODO: if(appsButton) xfdashboard_button_set_icon(XFDASHBOARD_BUTTON(appsButton), GTK_STOCK_FIND);
-		if(appsButton) xfdashboard_actor_add_style_class(XFDASHBOARD_ACTOR(appsButton), "search-active");
+		if(appsButton) xfdashboard_stylable_add_class(XFDASHBOARD_STYLABLE(appsButton), "search-active");
 
 		/* Emit "search-started" signal */
 		g_signal_emit(self, XfdashboardStageSignals[SIGNAL_SEARCH_STARTED], 0);
@@ -282,15 +493,13 @@ static void _xfdashboard_stage_on_searchbox_text_changed(XfdashboardStage *self,
 		}
 
 		/* Deactivate "clear" button on text box */
-		// TODO: xfdashboard_text_box_set_secondary_icon(XFDASHBOARD_TEXT_BOX(priv->searchbox), NULL);
-		xfdashboard_actor_remove_style_class(XFDASHBOARD_ACTOR(priv->searchbox), "search-active");
+		xfdashboard_stylable_remove_class(XFDASHBOARD_STYLABLE(priv->searchbox), "search-active");
 
 		/* Disable search view */
 		xfdashboard_view_set_enabled(searchView, FALSE);
 
 		/* Change apps button appearance */
-		// TODO: if(appsButton) xfdashboard_button_set_icon(XFDASHBOARD_BUTTON(appsButton), GTK_STOCK_HOME);
-		if(appsButton) xfdashboard_actor_remove_style_class(XFDASHBOARD_ACTOR(appsButton), "search-active");
+		if(appsButton) xfdashboard_stylable_remove_class(XFDASHBOARD_STYLABLE(appsButton), "search-active");
 
 		/* Emit "search-ended" signal */
 		g_signal_emit(self, XfdashboardStageSignals[SIGNAL_SEARCH_ENDED], 0);
@@ -359,124 +568,6 @@ static void _xfdashboard_stage_on_view_activated(XfdashboardStage *self, Xfdashb
 	}
 }
 
-/* Set up stage */
-static void _xfdashboard_stage_setup(XfdashboardStage *self)
-{
-	/* TODO: Implement missing actors, do setup nicer and themable/layoutable */
-	/* TODO: Create background by copying background of Xfce */
-
-	XfdashboardStagePrivate		*priv;
-	ClutterActor				*groupHorizontal;
-	ClutterActor				*groupVertical;
-	ClutterActor				*collapseBox;
-	ClutterLayoutManager		*layout;
-	XfdashboardToggleButton		*appsButton;
-
-	g_return_if_fail(XFDASHBOARD_IS_STAGE(self));
-
-	priv=self->priv;
-
-	/* Set up layout objects */
-	layout=clutter_box_layout_new();
-	clutter_box_layout_set_orientation(CLUTTER_BOX_LAYOUT(layout), CLUTTER_ORIENTATION_VERTICAL);
-	clutter_box_layout_set_spacing(CLUTTER_BOX_LAYOUT(layout), 8.0f);
-	clutter_box_layout_set_homogeneous(CLUTTER_BOX_LAYOUT(layout), FALSE);
-	groupVertical=clutter_actor_new();
-	clutter_actor_set_x_expand(groupVertical, TRUE);
-	clutter_actor_set_y_expand(groupVertical, TRUE);
-	clutter_actor_set_layout_manager(groupVertical, layout);
-
-	/* Searchbox and view selector */
-	layout=clutter_box_layout_new();
-	clutter_box_layout_set_orientation(CLUTTER_BOX_LAYOUT(layout), CLUTTER_ORIENTATION_HORIZONTAL);
-	clutter_box_layout_set_spacing(CLUTTER_BOX_LAYOUT(layout), 8.0f);
-	groupHorizontal=clutter_actor_new();
-	clutter_actor_set_x_expand(groupHorizontal, TRUE);
-	clutter_actor_set_layout_manager(groupHorizontal, layout);
-
-	priv->viewSelector=xfdashboard_view_selector_new();
-	clutter_actor_set_name(priv->viewSelector, "view-selector");
-	clutter_actor_add_child(groupHorizontal, priv->viewSelector);
-
-	priv->searchbox=xfdashboard_text_box_new();
-	clutter_actor_set_name(priv->searchbox, "searchbox");
-	clutter_actor_set_x_expand(priv->searchbox, TRUE);
-	xfdashboard_text_box_set_editable(XFDASHBOARD_TEXT_BOX(priv->searchbox), TRUE);
-	xfdashboard_text_box_set_hint_text(XFDASHBOARD_TEXT_BOX(priv->searchbox), _("Just type to search..."));
-	g_signal_connect_swapped(priv->searchbox, "text-changed", G_CALLBACK(_xfdashboard_stage_on_searchbox_text_changed), self);
-	g_signal_connect_swapped(priv->searchbox, "secondary-icon-clicked", G_CALLBACK(_xfdashboard_stage_on_searchbox_secondary_icon_clicked), self);
-	clutter_actor_add_child(groupHorizontal, priv->searchbox);
-
-	clutter_actor_add_child(groupVertical, groupHorizontal);
-
-	/* Views */
-	priv->viewpad=xfdashboard_viewpad_new();
-	clutter_actor_set_x_expand(priv->viewpad, TRUE);
-	clutter_actor_set_y_expand(priv->viewpad, TRUE);
-	clutter_actor_add_child(groupVertical, priv->viewpad);
-	g_signal_connect_swapped(priv->viewpad, "view-activated", G_CALLBACK(_xfdashboard_stage_on_view_activated), self);
-	xfdashboard_view_selector_set_viewpad(XFDASHBOARD_VIEW_SELECTOR(priv->viewSelector), XFDASHBOARD_VIEWPAD(priv->viewpad));
-
-	/* Set up layout objects */
-	layout=clutter_box_layout_new();
-	clutter_box_layout_set_orientation(CLUTTER_BOX_LAYOUT(layout), CLUTTER_ORIENTATION_HORIZONTAL);
-	clutter_box_layout_set_spacing(CLUTTER_BOX_LAYOUT(layout), 8.0f);
-	clutter_box_layout_set_homogeneous(CLUTTER_BOX_LAYOUT(layout), FALSE);
-	groupHorizontal=clutter_actor_new();
-	clutter_actor_set_x_expand(groupHorizontal, TRUE);
-	clutter_actor_set_y_expand(groupHorizontal, TRUE);
-	clutter_actor_set_layout_manager(groupHorizontal, layout);
-
-	/* Quicklaunch */
-	priv->quicklaunch=xfdashboard_quicklaunch_new_with_orientation(CLUTTER_ORIENTATION_VERTICAL);
-	clutter_actor_set_name(priv->quicklaunch, "quicklaunch");
-	clutter_actor_set_y_expand(priv->quicklaunch, TRUE);
-	clutter_actor_add_child(groupHorizontal, priv->quicklaunch);
-
-	appsButton=xfdashboard_quicklaunch_get_apps_button(XFDASHBOARD_QUICKLAUNCH(priv->quicklaunch));
-	if(appsButton)
-	{
-		g_signal_connect_swapped(appsButton, "toggled", G_CALLBACK(_xfdashboard_stage_on_quicklaunch_apps_button_toggled), self);
-	}
-
-	/* Set up layout objects */
-	clutter_actor_add_child(groupHorizontal, groupVertical);
-
-	/* Workspaces selector */
-	priv->workspaces=xfdashboard_workspace_selector_new();
-	clutter_actor_set_name(priv->workspaces, "workspace-selector");
-	clutter_actor_set_y_expand(priv->workspaces, TRUE);
-
-	collapseBox=xfdashboard_collapse_box_new();
-	clutter_actor_set_name(collapseBox, "workspace-selector-collapse-box");
-	clutter_actor_set_y_expand(collapseBox, TRUE);
-	clutter_actor_add_child(collapseBox, priv->workspaces);
-	clutter_actor_add_child(groupHorizontal, collapseBox);
-
-	/* Set up layout objects */
-	clutter_actor_add_constraint(groupHorizontal, clutter_bind_constraint_new(CLUTTER_ACTOR(self), CLUTTER_BIND_X, 0.0f));
-	clutter_actor_add_constraint(groupHorizontal, clutter_bind_constraint_new(CLUTTER_ACTOR(self), CLUTTER_BIND_Y, 8.0f));
-	clutter_actor_add_constraint(groupHorizontal, clutter_bind_constraint_new(CLUTTER_ACTOR(self), CLUTTER_BIND_WIDTH, 0.0f));
-	clutter_actor_add_constraint(groupHorizontal, clutter_bind_constraint_new(CLUTTER_ACTOR(self), CLUTTER_BIND_HEIGHT, -16.0f));
-	clutter_actor_add_child(CLUTTER_ACTOR(self), groupHorizontal);
-
-	/* Notification actor (this actor is above all others and hidden by default) */
-	priv->notification=xfdashboard_text_box_new();
-	clutter_actor_set_name(priv->notification, "notification");
-	clutter_actor_hide(priv->notification);
-	clutter_actor_set_reactive(priv->notification, FALSE);
-	clutter_actor_set_fixed_position_set(priv->notification, TRUE);
-	clutter_actor_set_z_position(priv->notification, 0.1f);
-	clutter_actor_set_request_mode(priv->notification, CLUTTER_REQUEST_HEIGHT_FOR_WIDTH);
-	clutter_actor_add_constraint(priv->notification, clutter_align_constraint_new(CLUTTER_ACTOR(self), CLUTTER_ALIGN_X_AXIS, 0.5f));
-	clutter_actor_add_constraint(priv->notification, clutter_align_constraint_new(groupHorizontal, CLUTTER_ALIGN_Y_AXIS, 1.0f));
-	clutter_actor_add_child(CLUTTER_ACTOR(self), priv->notification);
-
-	/* Set key focus to searchbox */
-	clutter_stage_set_accept_focus(CLUTTER_STAGE(self), TRUE);
-	clutter_stage_set_key_focus(CLUTTER_STAGE(self), priv->searchbox);
-}
-
 /* A window was created
  * Check for stage window and set up window properties
  */
@@ -486,9 +577,11 @@ static void _xfdashboard_stage_on_window_opened(XfdashboardStage *self,
 {
 	XfdashboardStagePrivate				*priv=self->priv;
 	XfdashboardWindowTrackerWindow		*stageWindow;
+#if !defined(CHECK_CLUTTER_VERSION) || !CLUTTER_CHECK_VERSION(1, 16, 0)
 	GdkScreen							*screen;
 	gint								primaryMonitor;
 	GdkRectangle						geometry;
+#endif
 
 	g_return_if_fail(XFDASHBOARD_IS_STAGE(self));
 	g_return_if_fail(XFDASHBOARD_IS_WINDOW_TRACKER_WINDOW(inWindow));
@@ -499,6 +592,11 @@ static void _xfdashboard_stage_on_window_opened(XfdashboardStage *self,
 	stageWindow=xfdashboard_window_tracker_window_get_stage_window(CLUTTER_STAGE(self));
 	if(stageWindow!=inWindow) return;
 
+	/* Set up window for use as stage window */
+	priv->stageWindow=inWindow;
+	xfdashboard_window_tracker_window_make_stage_window(priv->stageWindow);
+
+#if !defined(CHECK_CLUTTER_VERSION) || !CLUTTER_CHECK_VERSION(1, 16, 0)
 	/* TODO: As long as we do not support multi-monitors
 	 *       use this hack to ensure stage is in right size
 	 */
@@ -506,17 +604,14 @@ static void _xfdashboard_stage_on_window_opened(XfdashboardStage *self,
 	primaryMonitor=gdk_screen_get_primary_monitor(screen);
 	gdk_screen_get_monitor_geometry(screen, primaryMonitor, &geometry);
 	clutter_actor_set_size(CLUTTER_ACTOR(self), geometry.width, geometry.height);
-	xfdashboard_window_tracker_window_move_resize(stageWindow,
-													geometry.x, geometry.y,
-													geometry.width, geometry.height);
-
-	/* Set up window for use as stage window */
-	priv->stageWindow=inWindow;
-	xfdashboard_window_tracker_window_make_stage_window(priv->stageWindow);
+#endif
 
 	/* Disconnect signal handler as this is a one-time setup of stage window */
-	g_debug(_("Stage window was opened and set up. Removing signal handler."));
+	g_debug("Stage window was opened and set up. Removing signal handler.");
 	g_signal_handlers_disconnect_by_func(priv->windowTracker, G_CALLBACK(_xfdashboard_stage_on_window_opened), self);
+
+	/* Set focus */
+	_xfdashboard_stage_set_focus(self);
 }
 
 /* The application will be suspended */
@@ -542,6 +637,12 @@ static void _xfdashboard_stage_on_application_suspend(XfdashboardStage *self, gp
 		xfdashboard_window_tracker_window_unmake_stage_window(priv->stageWindow);
 		xfdashboard_window_tracker_window_hide(priv->stageWindow);
 	}
+
+	/* Hide tooltip and clean up */
+	if(priv->tooltip) clutter_actor_hide(priv->tooltip);
+	g_signal_handlers_disconnect_by_func(self,
+											G_CALLBACK(_xfdashboard_stage_on_captured_event_after_tooltip),
+											NULL);
 }
 
 /* The application will be resumed */
@@ -562,10 +663,19 @@ static void _xfdashboard_stage_on_application_resume(XfdashboardStage *self, gpo
 		doResetSearch=xfconf_channel_get_bool(xfdashboard_application_get_xfconf_channel(),
 												RESET_SEARCH_ON_RESUME_XFCONF_PROP,
 												DEFAULT_RESET_SEARCH_ON_RESUME);
+
 		if(priv->searchbox &&
 			doResetSearch &&
 			!xfdashboard_text_box_is_empty(XFDASHBOARD_TEXT_BOX(priv->searchbox)))
 		{
+			XfdashboardView				*searchView;
+
+			/* Reset search in search view */
+			searchView=xfdashboard_viewpad_find_view_by_type(XFDASHBOARD_VIEWPAD(priv->viewpad), XFDASHBOARD_TYPE_SEARCH_VIEW);
+			if(searchView) xfdashboard_search_view_reset_search(XFDASHBOARD_SEARCH_VIEW(searchView));
+				else g_critical(_("Cannot reset search because search view was not found in viewpad."));
+
+			/* Reset text in search box */
 			xfdashboard_text_box_set_text(XFDASHBOARD_TEXT_BOX(priv->searchbox), NULL);
 		}
 
@@ -579,6 +689,9 @@ static void _xfdashboard_stage_on_application_resume(XfdashboardStage *self, gpo
 			/* Show stage and force window creation */
 			clutter_actor_show(CLUTTER_ACTOR(self));
 		}
+
+	/* In any case force a redraw */
+	clutter_actor_queue_redraw(CLUTTER_ACTOR(self));
 }
 
 /* IMPLEMENTATION: ClutterActor */
@@ -611,6 +724,206 @@ static void _xfdashboard_stage_show(ClutterActor *inActor)
 	}
 }
 
+/* IMPLEMENTATION: Interface XfdashboardLayoutable */
+
+/* Virtual function "layout_completed" was called */
+static void _xfdashboard_stage_layoutable_layout_completed(XfdashboardLayoutable *inLayoutable)
+{
+	XfdashboardStage			*self;
+	XfdashboardStagePrivate		*priv;
+	ClutterActor				*actor;
+
+	g_return_if_fail(XFDASHBOARD_IS_LAYOUTABLE(inLayoutable));
+	g_return_if_fail(XFDASHBOARD_IS_STAGE(inLayoutable));
+
+	self=XFDASHBOARD_STAGE(inLayoutable);
+	priv=self->priv;
+
+	/* Get children from built stage and connect signals */
+	actor=xfdashboard_find_actor_by_name(CLUTTER_ACTOR(self), "view-selector");
+	if(actor && XFDASHBOARD_IS_VIEW_SELECTOR(actor))
+	{
+		priv->viewSelector=actor;
+
+		/* Register this focusable actor if it is focusable */
+		if(XFDASHBOARD_IS_FOCUSABLE(priv->viewSelector))
+		{
+			xfdashboard_focus_manager_register(priv->focusManager,
+												XFDASHBOARD_FOCUSABLE(priv->viewSelector));
+		}
+	}
+
+	actor=xfdashboard_find_actor_by_name(CLUTTER_ACTOR(self), "searchbox");
+	if(actor && XFDASHBOARD_IS_TEXT_BOX(actor))
+	{
+		priv->searchbox=actor;
+
+		/* Connect signals */
+		g_signal_connect_swapped(priv->searchbox,
+									"text-changed",
+									G_CALLBACK(_xfdashboard_stage_on_searchbox_text_changed),
+									self);
+		g_signal_connect_swapped(priv->searchbox,
+									"secondary-icon-clicked",
+									G_CALLBACK(_xfdashboard_stage_on_searchbox_secondary_icon_clicked),
+									self);
+
+		/* Register this focusable actor if it is focusable */
+		if(XFDASHBOARD_IS_FOCUSABLE(priv->searchbox))
+		{
+			xfdashboard_focus_manager_register(priv->focusManager,
+												XFDASHBOARD_FOCUSABLE(priv->searchbox));
+		}
+	}
+
+	actor=xfdashboard_find_actor_by_name(CLUTTER_ACTOR(self), "viewpad");
+	if(actor && XFDASHBOARD_IS_VIEWPAD(actor))
+	{
+		priv->viewpad=actor;
+
+		/* Connect signals */
+		g_signal_connect_swapped(priv->viewpad, "view-activated", G_CALLBACK(_xfdashboard_stage_on_view_activated), self);
+
+		/* Register this focusable actor if it is focusable */
+		if(XFDASHBOARD_IS_FOCUSABLE(priv->viewpad))
+		{
+			xfdashboard_focus_manager_register(priv->focusManager,
+												XFDASHBOARD_FOCUSABLE(priv->viewpad));
+		}
+	}
+
+	actor=xfdashboard_find_actor_by_name(CLUTTER_ACTOR(self), "quicklaunch");
+	if(actor && XFDASHBOARD_IS_QUICKLAUNCH(actor))
+	{
+		XfdashboardToggleButton		*appsButton;
+
+		priv->quicklaunch=actor;
+
+		/* Connect signals */
+		appsButton=xfdashboard_quicklaunch_get_apps_button(XFDASHBOARD_QUICKLAUNCH(priv->quicklaunch));
+		if(appsButton)
+		{
+			g_signal_connect_swapped(appsButton,
+										"toggled",
+										G_CALLBACK(_xfdashboard_stage_on_quicklaunch_apps_button_toggled),
+										self);
+		}
+
+		/* Register this focusable actor if it is focusable */
+		if(XFDASHBOARD_IS_FOCUSABLE(priv->quicklaunch))
+		{
+			xfdashboard_focus_manager_register(priv->focusManager,
+												XFDASHBOARD_FOCUSABLE(priv->quicklaunch));
+		}
+	}
+
+	actor=xfdashboard_find_actor_by_name(CLUTTER_ACTOR(self), "workspace-selector");
+	if(actor && XFDASHBOARD_IS_WORKSPACE_SELECTOR(actor))
+	{
+		priv->workspaces=actor;
+
+		/* Register this focusable actor if it is focusable */
+		if(XFDASHBOARD_IS_FOCUSABLE(priv->workspaces))
+		{
+			xfdashboard_focus_manager_register(priv->focusManager,
+												XFDASHBOARD_FOCUSABLE(priv->workspaces));
+		}
+	}
+
+	actor=xfdashboard_find_actor_by_name(CLUTTER_ACTOR(self), "notification");
+	if(actor && XFDASHBOARD_IS_TEXT_BOX(actor))
+	{
+		priv->notification=actor;
+
+		/* Register this focusable actor if it is focusable */
+		if(XFDASHBOARD_IS_FOCUSABLE(priv->notification))
+		{
+			xfdashboard_focus_manager_register(priv->focusManager,
+												XFDASHBOARD_FOCUSABLE(priv->notification));
+		}
+
+		/* Hide notification by default */
+		clutter_actor_hide(priv->notification);
+		clutter_actor_set_reactive(priv->notification, FALSE);
+	}
+
+	actor=xfdashboard_find_actor_by_name(CLUTTER_ACTOR(self), "tooltip");
+	if(actor && XFDASHBOARD_IS_TEXT_BOX(actor))
+	{
+		priv->tooltip=actor;
+
+		/* Register this focusable actor if it is focusable */
+		if(XFDASHBOARD_IS_FOCUSABLE(priv->tooltip))
+		{
+			xfdashboard_focus_manager_register(priv->focusManager,
+												XFDASHBOARD_FOCUSABLE(priv->tooltip));
+		}
+
+		/* Hide tooltip by default */
+		clutter_actor_hide(priv->tooltip);
+		clutter_actor_set_reactive(priv->tooltip, FALSE);
+	}
+
+	/* Set focus */
+	_xfdashboard_stage_set_focus(self);
+}
+
+/* Interface initialization
+ * Set up default functions
+ */
+void _xfdashboard_stage_layoutable_iface_init(XfdashboardLayoutableInterface *iface)
+{
+	iface->layout_completed=_xfdashboard_stage_layoutable_layout_completed;
+}
+
+/* IMPLEMENTATION: Interface XfdashboardStylable */
+
+/* Get stylable properties of stage */
+static void _xfdashboard_stage_stylable_get_stylable_properties(XfdashboardStylable *self,
+																GHashTable *ioStylableProperties)
+{
+	g_return_if_fail(XFDASHBOARD_IS_STYLABLE(self));
+
+	/* Add stylable properties to hashtable */
+	xfdashboard_stylable_add_stylable_property(self, ioStylableProperties, "background-color");
+}
+
+/* Get/set style classes of stage */
+static const gchar* _xfdashboard_stage_stylable_get_classes(XfdashboardStylable *inStylable)
+{
+	/* Not implemented */
+	return(NULL);
+}
+
+static void _xfdashboard_stage_stylable_set_classes(XfdashboardStylable *inStylable, const gchar *inStyleClasses)
+{
+	/* Not implemented */
+}
+
+/* Get/set style pseudo-classes of stage */
+static const gchar* _xfdashboard_stage_stylable_get_pseudo_classes(XfdashboardStylable *inStylable)
+{
+	/* Not implemented */
+	return(NULL);
+}
+
+static void _xfdashboard_stage_stylable_set_pseudo_classes(XfdashboardStylable *inStylable, const gchar *inStylePseudoClasses)
+{
+	/* Not implemented */
+}
+
+/* Interface initialization
+ * Set up default functions
+ */
+void _xfdashboard_stage_stylable_iface_init(XfdashboardStylableInterface *iface)
+{
+	iface->get_stylable_properties=_xfdashboard_stage_stylable_get_stylable_properties;
+	iface->get_classes=_xfdashboard_stage_stylable_get_classes;
+	iface->set_classes=_xfdashboard_stage_stylable_set_classes;
+	iface->get_pseudo_classes=_xfdashboard_stage_stylable_get_pseudo_classes;
+	iface->set_pseudo_classes=_xfdashboard_stage_stylable_set_pseudo_classes;
+}
+
 /* IMPLEMENTATION: GObject */
 
 /* Dispose this object */
@@ -626,6 +939,12 @@ static void _xfdashboard_stage_dispose(GObject *inObject)
 		priv->stageWindow=NULL;
 	}
 
+	if(priv->focusManager)
+	{
+		g_object_unref(priv->focusManager);
+		priv->focusManager=NULL;
+	}
+
 	if(priv->notificationTimeoutID)
 	{
 		g_source_remove(priv->notificationTimeoutID);
@@ -639,12 +958,30 @@ static void _xfdashboard_stage_dispose(GObject *inObject)
 		priv->windowTracker=NULL;
 	}
 
+	if(priv->styleClasses)
+	{
+		g_free(priv->styleClasses);
+		priv->styleClasses=NULL;
+	}
+
+	if(priv->stylePseudoClasses)
+	{
+		g_free(priv->stylePseudoClasses);
+		priv->stylePseudoClasses=NULL;
+	}
+
 	if(priv->notification)
 	{
 		clutter_actor_destroy(CLUTTER_ACTOR(priv->notification));
 		priv->notification=NULL;
 	}
 
+	if(priv->tooltip)
+	{
+		clutter_actor_destroy(CLUTTER_ACTOR(priv->tooltip));
+		priv->tooltip=NULL;
+	}
+
 	if(priv->quicklaunch)
 	{
 		clutter_actor_destroy(priv->quicklaunch);
@@ -685,6 +1022,56 @@ static void _xfdashboard_stage_dispose(GObject *inObject)
 	G_OBJECT_CLASS(xfdashboard_stage_parent_class)->dispose(inObject);
 }
 
+/* Set/get properties */
+static void _xfdashboard_stage_set_property(GObject *inObject,
+											guint inPropID,
+											const GValue *inValue,
+											GParamSpec *inSpec)
+{
+	XfdashboardStage			*self=XFDASHBOARD_STAGE(inObject);
+
+	switch(inPropID)
+	{
+		case PROP_STYLE_CLASSES:
+			_xfdashboard_stage_stylable_set_classes(XFDASHBOARD_STYLABLE(self),
+													g_value_get_string(inValue));
+			break;
+
+		case PROP_STYLE_PSEUDO_CLASSES:
+			_xfdashboard_stage_stylable_set_pseudo_classes(XFDASHBOARD_STYLABLE(self),
+															g_value_get_string(inValue));
+			break;
+
+		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID(inObject, inPropID, inSpec);
+			break;
+	}
+}
+
+static void _xfdashboard_stage_get_property(GObject *inObject,
+											guint inPropID,
+											GValue *outValue,
+											GParamSpec *inSpec)
+{
+	XfdashboardStage			*self=XFDASHBOARD_STAGE(inObject);
+	XfdashboardStagePrivate		*priv=self->priv;
+
+	switch(inPropID)
+	{
+		case PROP_STYLE_CLASSES:
+			g_value_set_string(outValue, priv->styleClasses);
+			break;
+
+		case PROP_STYLE_PSEUDO_CLASSES:
+			g_value_set_string(outValue, priv->stylePseudoClasses);
+			break;
+
+		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID(inObject, inPropID, inSpec);
+			break;
+	}
+}
+
 /* Class initialization
  * Override functions in parent classes and define properties
  * and signals
@@ -695,13 +1082,22 @@ static void xfdashboard_stage_class_init(XfdashboardStageClass *klass)
 	GObjectClass		*gobjectClass=G_OBJECT_CLASS(klass);
 
 	/* Override functions */
+	klass->show_tooltip=_xfdashboard_stage_show_tooltip;
+
 	actorClass->show=_xfdashboard_stage_show;
+	actorClass->event=xfdashboard_stage_event;
 
 	gobjectClass->dispose=_xfdashboard_stage_dispose;
+	gobjectClass->set_property=_xfdashboard_stage_set_property;
+	gobjectClass->get_property=_xfdashboard_stage_get_property;
 
 	/* Set up private structure */
 	g_type_class_add_private(klass, sizeof(XfdashboardStagePrivate));
 
+	/* Define properties */
+	g_object_class_override_property(gobjectClass, PROP_STYLE_CLASSES, "style-classes");
+	g_object_class_override_property(gobjectClass, PROP_STYLE_PSEUDO_CLASSES, "style-pseudo-classes");
+
 	/* Define signals */
 	XfdashboardStageSignals[SIGNAL_SEARCH_STARTED]=
 		g_signal_new("search-started",
@@ -736,6 +1132,18 @@ static void xfdashboard_stage_class_init(XfdashboardStageClass *klass)
 						g_cclosure_marshal_VOID__VOID,
 						G_TYPE_NONE,
 						0);
+
+	XfdashboardStageSignals[SIGNAL_SHOW_TOOLTIP]=
+		g_signal_new("show-tooltip",
+						G_TYPE_FROM_CLASS(klass),
+						G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+						G_STRUCT_OFFSET(XfdashboardStageClass, show_tooltip),
+						NULL,
+						NULL,
+						g_cclosure_marshal_VOID__OBJECT,
+						G_TYPE_NONE,
+						1,
+						CLUTTER_TYPE_ACTION);
 }
 
 /* Object initialization
@@ -745,12 +1153,20 @@ static void xfdashboard_stage_init(XfdashboardStage *self)
 {
 	XfdashboardStagePrivate		*priv;
 	XfdashboardApplication		*application;
+#if defined(CLUTTER_CHECK_VERSION) && CLUTTER_CHECK_VERSION(1, 16, 0)
+	GdkScreen					*screen;
+	gint						primaryMonitor;
+	GdkRectangle				geometry;
+#endif
 
 	priv=self->priv=XFDASHBOARD_STAGE_GET_PRIVATE(self);
 
 	/* Set default values */
+	priv->focusManager=xfdashboard_focus_manager_get_default();
 	priv->windowTracker=xfdashboard_window_tracker_get_default();
 	priv->stageWindow=NULL;
+	priv->styleClasses=NULL;
+	priv->stylePseudoClasses=NULL;
 
 	priv->quicklaunch=NULL;
 	priv->searchbox=NULL;
@@ -758,20 +1174,27 @@ static void xfdashboard_stage_init(XfdashboardStage *self)
 	priv->viewpad=NULL;
 	priv->viewSelector=NULL;
 	priv->notification=NULL;
+	priv->tooltip=NULL;
 	priv->lastSearchTextLength=0;
 	priv->viewBeforeSearch=NULL;
 	priv->searchActive=FALSE;
 	priv->notificationTimeoutID=0;
 
-	/* Set up stage */
-	clutter_actor_set_background_color(CLUTTER_ACTOR(self), &defaultStageColor);
+	/* Set up stage and style it */
 	clutter_stage_set_use_alpha(CLUTTER_STAGE(self), TRUE);
 	clutter_stage_set_user_resizable(CLUTTER_STAGE(self), FALSE);
 	clutter_stage_set_fullscreen(CLUTTER_STAGE(self), TRUE);
+	xfdashboard_stylable_invalidate(XFDASHBOARD_STYLABLE(self));
 
-	_xfdashboard_stage_setup(self);
-
-	g_signal_connect_swapped(self, "key-release-event", G_CALLBACK(_xfdashboard_stage_on_key_release), self);
+#if defined(CLUTTER_CHECK_VERSION) && CLUTTER_CHECK_VERSION(1, 16, 0)
+	/* TODO: As long as we do not support multi-monitors
+	 *       use this hack to ensure stage is in right size
+	 */
+	screen=gdk_screen_get_default();
+	primaryMonitor=gdk_screen_get_primary_monitor(screen);
+	gdk_screen_get_monitor_geometry(screen, primaryMonitor, &geometry);
+	clutter_actor_set_size(CLUTTER_ACTOR(self), geometry.width, geometry.height);
+#endif
 
 	/* Connect signal to application */
 	application=xfdashboard_application_get_default();
