@@ -27,17 +27,24 @@
 #endif
 
 #include "text-box.h"
-#include "enums.h"
-#include "button.h"
 
 #include <glib/gi18n-lib.h>
 #include <math.h>
 
+#include "enums.h"
+#include "button.h"
+#include "stylable.h"
+#include "focusable.h"
+#include "focus-manager.h"
+
 /* Define this class in GObject system */
-G_DEFINE_TYPE(XfdashboardTextBox,
-				xfdashboard_text_box,
-				XFDASHBOARD_TYPE_BACKGROUND)
-                                                
+static void _xfdashboard_text_box_focusable_iface_init(XfdashboardFocusableInterface *iface);
+
+G_DEFINE_TYPE_WITH_CODE(XfdashboardTextBox,
+						xfdashboard_text_box,
+						XFDASHBOARD_TYPE_BACKGROUND,
+						G_IMPLEMENT_INTERFACE(XFDASHBOARD_TYPE_FOCUSABLE, _xfdashboard_text_box_focusable_iface_init))
+
 /* Private structure - access only by public API if needed */
 #define XFDASHBOARD_TEXT_BOX_GET_PRIVATE(obj) \
 	(G_TYPE_INSTANCE_GET_PRIVATE((obj), XFDASHBOARD_TYPE_TEXT_BOX, XfdashboardTextBoxPrivate))
@@ -115,8 +122,6 @@ enum
 
 static guint XfdashboardTextBoxSignals[SIGNAL_LAST]={ 0, };
 
-/* Private constants */
-
 /* IMPLEMENTATION: Private variables and methods */
 
 /* Text of editable text box has changed */
@@ -173,10 +178,16 @@ static void _xfdashboard_text_box_key_focus_in(ClutterActor *inActor)
 	XfdashboardTextBox			*self=XFDASHBOARD_TEXT_BOX(inActor);
 	XfdashboardTextBoxPrivate	*priv=self->priv;
 	ClutterStage				*stage;
+	XfdashboardFocusManager		*focusManager;
 
 	/* Push key focus forward to text box */
 	stage=CLUTTER_STAGE(clutter_actor_get_stage(inActor));
 	clutter_stage_set_key_focus(stage, priv->actorTextBox);
+
+	/* Update focus in focus manager */
+	focusManager=xfdashboard_focus_manager_get_default();
+	xfdashboard_focus_manager_set_focus(focusManager, XFDASHBOARD_FOCUSABLE(self));
+	g_object_unref(focusManager);
 }
 
 /* Show all children of this one */
@@ -446,6 +457,142 @@ static void _xfdashboard_text_box_destroy(ClutterActor *self)
 	CLUTTER_ACTOR_CLASS(xfdashboard_text_box_parent_class)->destroy(self);
 }
 
+/* IMPLEMENTATION: Interface XfdashboardFocusable */
+
+/* Virtual function "can_focus" was called */
+static gboolean _xfdashboard_text_box_focusable_can_focus(XfdashboardFocusable *inFocusable)
+{
+	XfdashboardTextBox				*self;
+	XfdashboardTextBoxPrivate		*priv;
+	XfdashboardFocusableInterface	*selfIface;
+	XfdashboardFocusableInterface	*parentIface;
+
+	g_return_val_if_fail(XFDASHBOARD_IS_TEXT_BOX(inFocusable), CLUTTER_EVENT_PROPAGATE);
+
+	self=XFDASHBOARD_TEXT_BOX(inFocusable);
+	priv=self->priv;
+
+	/* Check if actor could be focused at all ... */
+	selfIface=XFDASHBOARD_FOCUSABLE_GET_IFACE(inFocusable);
+	parentIface=g_type_interface_peek_parent(selfIface);
+
+	if(parentIface && parentIface->can_focus)
+	{
+		if(!parentIface->can_focus(inFocusable)) return(FALSE);
+	}
+
+	/* ... then only an editable text box can be focused */
+	if(priv->isEditable) return(TRUE);
+
+	/* If we get here this actor cannot be focused */
+	return(FALSE);
+}
+
+/* Virtual function "handle_key_event" was called */
+static gboolean _xfdashboard_text_box_focusable_handle_key_event(XfdashboardFocusable *inFocusable,
+																	const ClutterEvent *inEvent)
+{
+	XfdashboardTextBox			*self;
+	XfdashboardTextBoxPrivate	*priv;
+	gboolean					result;
+
+	g_return_val_if_fail(XFDASHBOARD_IS_TEXT_BOX(inFocusable), CLUTTER_EVENT_PROPAGATE);
+
+	self=XFDASHBOARD_TEXT_BOX(inFocusable);
+	priv=self->priv;
+	result=CLUTTER_EVENT_PROPAGATE;
+
+	/* Push event to real text box if available */
+	if(priv->actorTextBox)
+	{
+		result=clutter_actor_event(priv->actorTextBox, inEvent, FALSE);
+	}
+
+	/* Return event handling result */
+	return(result);
+}
+
+/* Set focus to actor */
+static void _xfdashboard_text_box_focusable_set_focus(XfdashboardFocusable *inFocusable)
+{
+	XfdashboardTextBox				*self;
+	XfdashboardTextBoxPrivate		*priv;
+	XfdashboardFocusableInterface	*selfIface;
+	XfdashboardFocusableInterface	*parentIface;
+	ClutterStage					*stage;
+
+	g_return_if_fail(XFDASHBOARD_IS_FOCUSABLE(inFocusable));
+	g_return_if_fail(XFDASHBOARD_IS_TEXT_BOX(inFocusable));
+
+	self=XFDASHBOARD_TEXT_BOX(inFocusable);
+	priv=self->priv;
+
+	/* Call parent class interface function */
+	selfIface=XFDASHBOARD_FOCUSABLE_GET_IFACE(inFocusable);
+	parentIface=g_type_interface_peek_parent(selfIface);
+
+	if(parentIface && parentIface->set_focus)
+	{
+		parentIface->set_focus(inFocusable);
+	}
+
+	/* Get stage of actor to tell it where the keyboard focus to set to */
+	stage=CLUTTER_STAGE(clutter_actor_get_stage(CLUTTER_ACTOR(self)));
+	if(!stage)
+	{
+		g_warning(_("Focusable actor %s is not on a stage"),
+					G_OBJECT_TYPE_NAME(self));
+		return;
+	}
+
+	clutter_stage_set_key_focus(stage, CLUTTER_ACTOR(priv->actorTextBox));
+}
+
+/* Unset focus from actor */
+static void _xfdashboard_text_box_focusable_unset_focus(XfdashboardFocusable *inFocusable)
+{
+	ClutterActor					*self;
+	XfdashboardFocusableInterface	*selfIface;
+	XfdashboardFocusableInterface	*parentIface;
+	ClutterStage					*stage;
+
+	g_return_if_fail(XFDASHBOARD_IS_FOCUSABLE(inFocusable));
+	g_return_if_fail(XFDASHBOARD_IS_ACTOR(inFocusable));
+
+	self=CLUTTER_ACTOR(inFocusable);
+
+	/* Call parent class interface function */
+	selfIface=XFDASHBOARD_FOCUSABLE_GET_IFACE(inFocusable);
+	parentIface=g_type_interface_peek_parent(selfIface);
+
+	if(parentIface && parentIface->unset_focus)
+	{
+		parentIface->unset_focus(inFocusable);
+	}
+
+	/* Get stage of actor to tell it where the keyboard focus to set to */
+	stage=CLUTTER_STAGE(clutter_actor_get_stage(self));
+	if(!stage)
+	{
+		g_warning(_("Focusable actor %s is not on a stage"),
+					G_OBJECT_TYPE_NAME(self));
+		return;
+	}
+
+	clutter_stage_set_key_focus(stage, NULL);
+}
+
+/* Interface initialization
+ * Set up default functions
+ */
+void _xfdashboard_text_box_focusable_iface_init(XfdashboardFocusableInterface *iface)
+{
+	iface->can_focus=_xfdashboard_text_box_focusable_can_focus;
+	iface->set_focus=_xfdashboard_text_box_focusable_set_focus;
+	iface->unset_focus=_xfdashboard_text_box_focusable_unset_focus;
+	iface->handle_key_event=_xfdashboard_text_box_focusable_handle_key_event;
+}
+
 /* IMPLEMENTATION: GObject */
 
 /* Dispose this object */
@@ -675,7 +822,7 @@ static void xfdashboard_text_box_class_init(XfdashboardTextBoxClass *klass)
 							_("Padding between background and elements"),
 							0.0f, G_MAXFLOAT,
 							0.0f,
-							G_PARAM_READWRITE);
+							G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
 	XfdashboardTextBoxProperties[PROP_SPACING]=
 		g_param_spec_float("spacing",
@@ -683,49 +830,49 @@ static void xfdashboard_text_box_class_init(XfdashboardTextBoxClass *klass)
 							_("Spacing between text and icon"),
 							0.0f, G_MAXFLOAT,
 							0.0f,
-							G_PARAM_READWRITE);
+							G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
 	XfdashboardTextBoxProperties[PROP_EDITABLE]=
 		g_param_spec_boolean("editable",
 								_("Editable"),
 								_("Flag to set if text is editable"),
 								FALSE,
-								G_PARAM_READWRITE);
+								G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
 	XfdashboardTextBoxProperties[PROP_PRIMARY_ICON_NAME]=
 		g_param_spec_string("primary-icon-name",
 							_("Primary icon name"),
 							_("Themed icon name or file name of primary icon shown left of text box"),
 							NULL,
-							G_PARAM_READWRITE);
+							G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
 	XfdashboardTextBoxProperties[PROP_SECONDARY_ICON_NAME]=
 		g_param_spec_string("secondary-icon-name",
 							_("Secondary icon name"),
 							_("Themed icon name or file name of secondary icon shown right of text box"),
 							NULL,
-							G_PARAM_READWRITE);
+							G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
 	XfdashboardTextBoxProperties[PROP_TEXT]=
 		g_param_spec_string("text",
 							_("Text"),
 							_("Text of editable text box"),
 							N_(""),
-							G_PARAM_READWRITE);
+							G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
 	XfdashboardTextBoxProperties[PROP_TEXT_FONT]=
 		g_param_spec_string("text-font",
 							_("Text font"),
 							_("Font of editable text box"),
 							NULL,
-							G_PARAM_READWRITE);
+							G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
 	XfdashboardTextBoxProperties[PROP_TEXT_COLOR]=
 		clutter_param_spec_color("text-color",
 									_("Text color"),
 									_("Color of text in editable text box"),
 									NULL,
-									G_PARAM_READWRITE);
+									G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
 
 	XfdashboardTextBoxProperties[PROP_SELECTION_TEXT_COLOR]=
@@ -733,35 +880,35 @@ static void xfdashboard_text_box_class_init(XfdashboardTextBoxClass *klass)
 									_("Selection text color"),
 									_("Color of text of selected text"),
 									NULL,
-									G_PARAM_READWRITE);
+									G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
 	XfdashboardTextBoxProperties[PROP_SELECTION_BACKGROUND_COLOR]=
 		clutter_param_spec_color("selection-background-color",
 									_("Selection background color"),
 									_("Color of background of selected text"),
 									NULL,
-									G_PARAM_READWRITE);
+									G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
 	XfdashboardTextBoxProperties[PROP_HINT_TEXT]=
 		g_param_spec_string("hint-text",
 							_("Hint text"),
 							_("Hint text shown if editable text box is empty"),
 							NULL,
-							G_PARAM_READWRITE);
+							G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
 	XfdashboardTextBoxProperties[PROP_HINT_TEXT_FONT]=
 		g_param_spec_string("hint-text-font",
 							_("Hint text font"),
 							_("Font of hint text shown if editable text box is empty"),
 							NULL,
-							G_PARAM_READWRITE);
+							G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
 	XfdashboardTextBoxProperties[PROP_HINT_TEXT_COLOR]=
 		clutter_param_spec_color("hint-text-color",
 									_("Hint text color"),
 									_("Color of hint text shown if editable text box is empty"),
 									NULL,
-									G_PARAM_READWRITE);
+									G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
 	g_object_class_install_properties(gobjectClass, PROP_LAST, XfdashboardTextBoxProperties);
 
@@ -842,14 +989,14 @@ static void xfdashboard_text_box_init(XfdashboardTextBox *self)
 
 	/* Create actors */
 	priv->actorPrimaryIcon=xfdashboard_button_new();
-	xfdashboard_actor_add_style_class(XFDASHBOARD_ACTOR(priv->actorPrimaryIcon), "primary-icon");
+	xfdashboard_stylable_add_class(XFDASHBOARD_STYLABLE(priv->actorPrimaryIcon), "primary-icon");
 	clutter_actor_set_reactive(priv->actorPrimaryIcon, TRUE);
 	clutter_actor_hide(priv->actorPrimaryIcon);
 	clutter_actor_add_child(CLUTTER_ACTOR(self), priv->actorPrimaryIcon);
 	g_signal_connect_swapped(priv->actorPrimaryIcon, "clicked", G_CALLBACK(_xfdashboard_text_box_on_primary_icon_clicked), self);
 
 	priv->actorSecondaryIcon=xfdashboard_button_new();
-	xfdashboard_actor_add_style_class(XFDASHBOARD_ACTOR(priv->actorSecondaryIcon), "secondary-icon");
+	xfdashboard_stylable_add_class(XFDASHBOARD_STYLABLE(priv->actorSecondaryIcon), "secondary-icon");
 	clutter_actor_set_reactive(priv->actorSecondaryIcon, TRUE);
 	clutter_actor_hide(priv->actorSecondaryIcon);
 	clutter_actor_add_child(CLUTTER_ACTOR(self), priv->actorSecondaryIcon);
@@ -957,8 +1104,8 @@ void xfdashboard_text_box_set_editable(XfdashboardTextBox *self, gboolean isEdit
 	if(priv->isEditable!=isEditable)
 	{
 		priv->isEditable=isEditable;
-		if(priv->isEditable) xfdashboard_actor_add_style_pseudo_class(XFDASHBOARD_ACTOR(self), "editable");
-			else xfdashboard_actor_remove_style_pseudo_class(XFDASHBOARD_ACTOR(self), "editable");
+		if(priv->isEditable) xfdashboard_stylable_add_pseudo_class(XFDASHBOARD_STYLABLE(self), "editable");
+			else xfdashboard_stylable_remove_pseudo_class(XFDASHBOARD_STYLABLE(self), "editable");
 
 		/* Set up actors */
 		clutter_text_set_selectable(CLUTTER_TEXT(priv->actorTextBox), priv->isEditable);
@@ -1044,6 +1191,25 @@ void xfdashboard_text_box_set_text(XfdashboardTextBox *self, const gchar *inMark
 	}
 }
 
+void xfdashboard_text_box_set_text_va(XfdashboardTextBox *self, const gchar *inFormat, ...)
+{
+	gchar						*text;
+	va_list						args;
+
+	g_return_if_fail(XFDASHBOARD_IS_TEXT_BOX(self));
+
+	/* Build formatted text to set */
+	va_start(args, inFormat);
+	text=g_strdup_vprintf(inFormat, args);
+	va_end(args);
+
+	/* Set formatted text */
+	xfdashboard_text_box_set_text(self, text);
+
+	/* Release allocated resources */
+	g_free(text);
+}
+
 /* Get/set font of editable text box */
 const gchar* xfdashboard_text_box_get_text_font(XfdashboardTextBox *self)
 {
@@ -1294,6 +1460,25 @@ void xfdashboard_text_box_set_hint_text(XfdashboardTextBox *self, const gchar *i
 	}
 }
 
+void xfdashboard_text_box_set_hint_text_va(XfdashboardTextBox *self, const gchar *inFormat, ...)
+{
+	gchar						*text;
+	va_list						args;
+
+	g_return_if_fail(XFDASHBOARD_IS_TEXT_BOX(self));
+
+	/* Build formatted text to set */
+	va_start(args, inFormat);
+	text=g_strdup_vprintf(inFormat, args);
+	va_end(args);
+
+	/* Set formatted text */
+	xfdashboard_text_box_set_hint_text(self, text);
+
+	/* Release allocated resources */
+	g_free(text);
+}
+
 /* Get/set font of hint label */
 const gchar* xfdashboard_text_box_get_hint_text_font(XfdashboardTextBox *self)
 {
