@@ -32,6 +32,7 @@
 
 #include "marshal.h"
 #include "image.h"
+#include "utils.h"
 
 /* Define this class in GObject system */
 G_DEFINE_ABSTRACT_TYPE(XfdashboardView,
@@ -95,6 +96,7 @@ enum
 	SIGNAL_ICON_CHANGED,
 
 	SIGNAL_SCROLL_TO,
+	SIGNAL_ENSURE_VISIBLE,
 
 	SIGNAL_LAST
 };
@@ -233,35 +235,35 @@ static void xfdashboard_view_class_init(XfdashboardViewClass *klass)
 							_("View internal name"),
 							_("Internal and untranslated name of view used in application"),
 							NULL,
-							G_PARAM_READWRITE);
+							G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
 	XfdashboardViewProperties[PROP_VIEW_NAME]=
 		g_param_spec_string("view-name",
 							_("View name"),
 							_("Name of view used to display"),
 							NULL,
-							G_PARAM_READWRITE);
+							G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
 	XfdashboardViewProperties[PROP_VIEW_ICON]=
 		g_param_spec_string("view-icon",
 							_("View icon"),
 							_("Icon of view used to display. Icon name can be a themed icon name or file name"),
 							NULL,
-							G_PARAM_READWRITE);
+							G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
 	XfdashboardViewProperties[PROP_FIT_MODE]=
 		g_param_spec_string("fit-mode",
 							_("Fit mode"),
 							_("Defines if view should be fit into viewpad and its orientation"),
 							XFDASHBOARD_FIT_MODE_NONE,
-							G_PARAM_READWRITE);
+							G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
 	XfdashboardViewProperties[PROP_ENABLED]=
 		g_param_spec_boolean("enabled",
 								_("Enabled"),
 								_("This flag indicates if is view is enabled and activable"),
 								TRUE,
-								G_PARAM_READABLE);
+								G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
 	g_object_class_install_properties(gobjectClass, PROP_LAST, XfdashboardViewProperties);
 
@@ -390,6 +392,18 @@ static void xfdashboard_view_class_init(XfdashboardViewClass *klass)
 						2,
 						G_TYPE_FLOAT,
 						G_TYPE_FLOAT);
+
+	XfdashboardViewSignals[SIGNAL_ENSURE_VISIBLE]=
+		g_signal_new("ensure-visible",
+						G_TYPE_FROM_CLASS(klass),
+						G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+						G_STRUCT_OFFSET(XfdashboardViewClass, ensure_visible),
+						NULL,
+						NULL,
+						g_cclosure_marshal_VOID__OBJECT,
+						G_TYPE_NONE,
+						1,
+						CLUTTER_TYPE_ACTOR);
 }
 
 /* Object initialization
@@ -565,8 +579,6 @@ void xfdashboard_view_set_enabled(XfdashboardView *self, gboolean inIsEnabled)
 		g_signal_emit(self, XfdashboardViewSignals[signalBeforeID], 0, self);
 
 		priv->isEnabled=inIsEnabled;
-		if(priv->isEnabled) xfdashboard_actor_add_style_pseudo_class(XFDASHBOARD_ACTOR(self), "enabled");
-			else xfdashboard_actor_remove_style_pseudo_class(XFDASHBOARD_ACTOR(self), "enabled");
 
 		g_signal_emit(self, XfdashboardViewSignals[signalAfterID], 0, self);
 
@@ -582,3 +594,16 @@ void xfdashboard_view_scroll_to(XfdashboardView *self, gfloat inX, gfloat inY)
 
 	g_signal_emit(self, XfdashboardViewSignals[SIGNAL_SCROLL_TO], 0, inX, inY);
 }
+
+/* Ensure that a child of this view is visible by scrolling if needed */
+void xfdashboard_view_ensure_visible(XfdashboardView *self, ClutterActor *inActor)
+{
+	g_return_if_fail(XFDASHBOARD_IS_VIEW(self));
+	g_return_if_fail(CLUTTER_IS_ACTOR(inActor));
+
+	/* Only emit signal if given actor is a child of this view */
+	if(xfdashboard_actor_contains_child_deep(CLUTTER_ACTOR(self), inActor))
+	{
+		g_signal_emit(self, XfdashboardViewSignals[SIGNAL_ENSURE_VISIBLE], 0, inActor);
+	}
+}
