@@ -311,7 +311,7 @@ static void _xfdashboard_window_content_release_resources(XfdashboardWindowConte
 	trapError=clutter_x11_untrap_x_errors();
 	if(trapError!=0)
 	{
-		g_debug(_("X error %d occured while releasing resources for window '%s"), trapError, xfdashboard_window_tracker_window_get_title(priv->window));
+		g_debug("X error %d occured while releasing resources for window '%s", trapError, xfdashboard_window_tracker_window_get_title(priv->window));
 		return;
 	}
 
@@ -374,7 +374,7 @@ static void _xfdashboard_window_content_suspend(XfdashboardWindowContent *self)
 	trapError=clutter_x11_untrap_x_errors();
 	if(trapError!=0)
 	{
-		g_debug(_("X error %d occured while suspending '%s"), trapError, xfdashboard_window_tracker_window_get_title(priv->window));
+		g_debug("X error %d occured while suspending '%s", trapError, xfdashboard_window_tracker_window_get_title(priv->window));
 		return;
 	}
 
@@ -435,7 +435,7 @@ static void _xfdashboard_window_content_resume(XfdashboardWindowContent *self)
 			/* Creating texture may fail if window is _NOT_ on active workspace
 			 * so display error message just as debug message (this time)
 			 */
-			g_debug(_("Could not create texture for window '%s': %s"),
+			g_debug("Could not create texture for window '%s': %s",
 						xfdashboard_window_tracker_window_get_title(priv->window),
 						error ? error->message : _("Unknown error"));
 			if(error)
@@ -504,7 +504,7 @@ static void _xfdashboard_window_content_resume(XfdashboardWindowContent *self)
 	trapError=clutter_x11_untrap_x_errors();
 	if(trapError!=0)
 	{
-		g_debug(_("X error %d occured while resuming window '%s"), trapError, xfdashboard_window_tracker_window_get_title(priv->window));
+		g_debug("X error %d occured while resuming window '%s", trapError, xfdashboard_window_tracker_window_get_title(priv->window));
 		return;
 	}
 
@@ -913,21 +913,21 @@ void xfdashboard_window_content_class_init(XfdashboardWindowContentClass *klass)
 							_("Window"),
 							_("The window to handle and display"),
 							XFDASHBOARD_TYPE_WINDOW_TRACKER_WINDOW,
-							G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
+							G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_CONSTRUCT_ONLY);
 
 	XfdashboardWindowContentProperties[PROP_SUSPENDED]=
 		g_param_spec_boolean("suspended",
 							_("Suspended"),
 							_("Is this window suspended"),
 							TRUE,
-							G_PARAM_READABLE);
+							G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
 	XfdashboardWindowContentProperties[PROP_OUTLINE_COLOR]=
 		clutter_param_spec_color("outline-color",
 									_("Outline color"),
 									_("Color to draw outline of mapped windows with"),
 									CLUTTER_COLOR_Black,
-									G_PARAM_READWRITE);
+									G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
 	XfdashboardWindowContentProperties[PROP_OUTLINE_WIDTH]=
 		g_param_spec_float("outline-width",
@@ -935,7 +935,7 @@ void xfdashboard_window_content_class_init(XfdashboardWindowContentClass *klass)
 							_("Width of line used to draw outline of mapped windows"),
 							0.0f, G_MAXFLOAT,
 							1.0f,
-							G_PARAM_READWRITE);
+							G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
 	g_object_class_install_properties(gobjectClass, PROP_LAST, XfdashboardWindowContentProperties);
 }
